import regex as re


WARNING_MESSAGE_RE = re.compile(
    r"""
^\ \*{3}\ WARNING\ in\ (?<filename>[^:]+):(?P<line>\d+)\ ::\ (?P<message>.+?) \*{3} \n
(\ \*{3}\ (?P<message>.+?) \*{3} \n)*
""",
    re.VERSION1 | re.VERBOSE | re.MULTILINE,
)

TOTAL_WARNING_COUNT_RE = re.compile(
    r"""
^\ The\ number\ of\ warnings\ for\ this\ run\ is\ :\s* (?P<value>\d+)
""",
    re.VERBOSE | re.MULTILINE,
)


def match_warnings(content):
    result = {"warnings": []}

    for wmatch in WARNING_MESSAGE_RE.finditer(content):
        result["warnings"] += [
            {"filename": wmatch["filename"], "line": int(wmatch["line"]), "message": "".join(wmatch.captures("message")).rstrip()}
        ]

    match = TOTAL_WARNING_COUNT_RE.search(content)
    if match:
        result["nwarnings"] = int(match["value"])

    return result



from typing import List

from .blocks import builtin_matchers
from .blocks.common import safe_string


def _mangled_keys(idict, mangle_func):
    if isinstance(idict, dict):
        return {mangle_func(k): _mangled_keys(v, mangle_func) for k, v in idict.items()}

    if isinstance(idict, list):
        return [_mangled_keys(i, mangle_func) for i in idict]

    return idict


def parse_iter(content: str, matchers: List[callable] = builtin_matchers, key_mangling: bool = False):
    for matcher in matchers:
        match = matcher(content)

        if match:
            if key_mangling:
                yield _mangled_keys(match, safe_string)
            else:
                yield match
